
{*******************************************************}
{                                                       }
{       StartBtn Renamer v2.02                          }
{       MainUnit Interface Unit [MainForm]              }
{                                                       }
{       Copyright  2002 AmirBehzad ESLAMI              }
{                                                       }
{*******************************************************}

unit MainUnit;

interface

uses
  Windows, Messages, Graphics, Controls,
  Classes, StdCtrls, ExtCtrls, SysUtils, Forms;

type
  TMainForm = class(TForm)
    Panel    : TPanel;
    Logo     : TImage;
    Bevel    : TBevel;
    Label1   : TLabel;
    Label2   : TLabel;
    Edit1    : TEdit;
    Edit2    : TEdit;
    Button   : TButton;
    About    : TLabel;
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonClick(Sender: TObject);
    procedure Edit2KeyPress(Sender: TObject; var Key: Char);
    procedure AboutClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation
{$R *.DFM}
{$R MoreRes.res MoreRes.rc}

uses
  MMSystem; // MMSystem Unit uses for PlaySound function

const
  AboutText = ' v2.02'#10#10 +
    'Changes Label of the [Start] button with your desirable text.'#10 +
    'Requires Windows XP. This program is Freeware and OpenSource.'#10#10 +
    'Written by AmirBehzad ESLAMI'#10#10 +
    'Web Site :  http://www.startbtn.main-page.com'#10 +
    'E-Mail      :  ab_eslami@worldsites.com';

  NeedXP = 'StartBtn Renamer only works under Windows XP.'#10 +
   'but you are using another version of Windows (9x/2000/ME/NT).';

const
  MaxLetters = 255;

  
type
  TStr = array [1..MaxLetters] of Char;

var
  HStartBtn: THandle;
  OldLabel, NewLabel: TStr;


{ GetStartHandle returns Handle of the [Start] button. }
function GetStartHandle: THandle;
begin
  Result := FindWindowEx(FindWindow('Shell_TrayWnd', nil), 0, 'Button', nil);
end;

{ GetStartLebel returns the current Label of the [Start] button, StartHandle
  parameter is the Handle of the [Start] button. Yoy can use the GetStartHandle
  function to get handle of the [Start] button. }
function GetStartLabel(StartHandle: THandle): TStr;
begin
  GetWindowText(StartHandle, @Result, MaxLetters + 1);
end;

{ SetStartLabel renames the [Start] button with the given new Label, which
  must be 255 letters. StartHandle is the Handle of the [Start] button. You can
  Use the GetStartHandle function to get handle of the [Start] button. }
procedure SetStartLabel(StartHandle: THandle; NewLabel: TStr);
const
  StartBtnWidth  = 99;
  StartBtnHeight = 32;
var
  ReBarWindow32Handle: THandle;
  ReBarWindow32Rect: TRect;
  TrayNotifyWndRect: TRect;
  TaskBarHandle: THandle;
  TaskBarRect: TRect;
  NewLabelWidth: Word;
  NewReBarWindow32Width: Word;
begin
  // Get TaskBar's Handle and Rectangle
  TaskBarHandle := FindWindow('Shell_TrayWnd', nil);
  GetWindowRect(TaskBarHandle, TaskBarRect);

  // Get ReBarWindow32's Handle and Rectangle (The place of Minimized programs)
  ReBarWindow32Handle := FindWindowEx(TaskBarHandle, 0, 'ReBarWindow32', nil);
  GetWindowRect(ReBarWindow32Handle, ReBarWindow32Rect);

  // Get TrayNotifyWnd's Rectangle (The place of Clock & Tray Icons)
  GetWindowRect(FindWindowEx(TaskBarHandle, 0, 'TrayNotifyWnd', nil),
    TrayNotifyWndRect);

  // Calculate Width size of NewLabel in pixels
  NewLabelWidth := MainForm.Canvas.TextWidth(TrimRight(NewLabel)) + 64;
  if NewLabelWidth < StartBtnWidth then
    NewLabelWidth := StartBtnWidth;

  NewReBarWindow32Width := (TaskBarRect.Right - TaskBarRect.Left) -
    (TrayNotifyWndRect.Right - TrayNotifyWndRect.Left) - NewLabelWidth;

  // Set new Label of the [Start] button
  SendMessage(StartHandle, WM_SETTEXT, 0, Integer(@NewLabel));
  // Resize the ReBarWindow32
  SetWindowPos(ReBarWindow32Handle, 0, NewLabelWidth + 5, 0,
    NewReBarWindow32Width, TaskBarRect.Bottom - TaskBarRect.Top, SWP_SHOWWINDOW);
  // Resize the [Start] button
  SetWindowPos(StartHandle, 0, 0, 0, NewLabelWidth, StartBtnHeight,
    SWP_SHOWWINDOW);
end;

{ IsWinXP returns Ture if platform is Windows XP, otherwise it returns False. }
function IsWinXP: Boolean;
begin
  Result := False;
  if Win32Platform = VER_PLATFORM_WIN32_NT then
    if (Win32MajorVersion > 5) or
      ((Win32MajorVersion = 5) and (Win32MinorVersion >= 1)) then
      Result := True;
end;


procedure TMainForm.FormCreate(Sender: TObject);
var
  i, Param1Len: Word;
begin
  Caption := Application.Title;
  if (not IsWinXP) then
  begin
    MessageBox(Handle, NeedXP, PChar(Application.Title), MB_ICONWARNING);
    Halt;
  end;
  HStartBtn := GetStartHandle;
  OldLabel := GetStartLabel(HStartBtn);
  if ParamCount > 0 then
  begin
    { Command-Line Running }
    Param1Len := Length(ParamStr(1));
    if Param1Len > MaxLetters then
      Param1Len := MaxLetters;
    for i := 1 to Param1Len do
      NewLabel[i] := ParamStr(1)[i];
    SetStartLabel(HStartBtn, NewLabel);
    Halt;
  end;
  Edit1.MaxLength := MaxLetters;
  Edit2.MaxLength := MaxLetters;
  Edit1.Text := TrimRight(OldLabel);
end;

procedure TMainForm.FormPaint(Sender: TObject);
var
  X, Y: Byte;
  BgBMP: TBitmap;
begin
  { Tile BgBMP on the MainForm }
  BgBMP := TBitmap.Create;
  try
    BgBMP.LoadFromResourceName(HInstance, 'ResBMP_1');
  { BgBMP.Handle := LoadBitmap(HInstance, 'ResBMP_1'); }
    for X := 0 to (Width div BgBMP.Width) + 1 do
      for Y := 0 to (Height div BgBMP.Height) + 1 do
        Canvas.Draw(X * BgBMP.Width, Y * BgBMP.Height, BgBMP);
  finally
    BgBMP.Free;
  end;
end;

procedure TMainForm.ButtonClick(Sender: TObject);
var
  i: Word;
begin
  with Edit2 do
  begin
    Text := Text + StringOfChar(' ', MaxLetters - Length(Text));
    for i := 1 to MaxLetters do
      NewLabel[i] := Text[i];
  end;
  PlaySound('ResWAV_1', HInstance, SND_ASYNC + SND_RESOURCE + SND_NODEFAULT);
  SetStartLabel(HStartBtn, NewLabel);
  MainForm.DoCreate;
  Edit2.SetFocus;
  Edit2.Clear;
end;

procedure TMainForm.Edit2KeyPress(Sender: TObject; var Key: Char);
begin
  if Key = Char(VK_RETURN) then
  begin
    Key := #00;  // This will disable the Beep sound
    Button.Click;
  end;
end;

procedure TMainForm.AboutClick(Sender: TObject);
begin
  MessageBox(Handle, PChar(Application.Title + AboutText), 'About',
    MB_APPLMODAL);
end;

end.
